<?php

namespace App\Service;

use App\Exception;

class ProductService {

    private $util;
    private $ProductFactory;

    public function __construct (\App\Util\Util $util,
                                 \App\Factory\ProductFactory $ProductFactory) {
        $this->util = $util;
        $this->productFactory  = $ProductFactory;
    }
    
    const CST_ICMS_SN = ['101', '102', '103', '300', '400', '500', '900'];
    
    public function getItensVenda($subsidiaryCode, $cdCaixa , $nrSeqVenda) {
        
        $itens = $this->productFactory->getItensVenda($subsidiaryCode, $cdCaixa , $nrSeqVenda);
        $arrayInfo = [];
        
        foreach($itens as $item) {
                $itemInfo = [
                        'id'              => $item['NRSEQUITVEND'],
                        'name'            => $item['NMPRODUTO'],
                        'code'            => $item['CDARVPROD'],
                        'externalCode'    => $item['CDPRODINTE'],
                        'quantity'        => $item['QTPRODVEND'],
                        'price'           => $this->util->tofloat($item['VRUNITVEND']),
                        'discount'        => $this->util->tofloat($item['VRDESITVEND']),
                        'addition'        => $this->util->tofloat($item['VRACRITVEND']),
                        'subsidizedValue' => $this->util->tofloat($item['VRUNITVENDCL']),
                        'total'           => $this->util->tofloat(bcadd($this->util->tofloat($item['VRUNITVENDCL']), $this->util->tofloat($item['TOTAL']), 2)),
                        'fiscalInfo'      => [
                                               'ncm'          => $item['CDCLASFISC'],
                                               'cest'         => $item['CDCEST'],
                                               'cstIcms'      => $item['CDCSTPRODI'],
                                               'cstPisCofins' => $item['CDCSTPRODPC'],
                                               'cfop'         => $item['CDCFOPPROD'],
                                               'bcPis'        => $this->util->tofloat($item['VRBCPISCOFINS']),
                                               'vrAliqPis'    => $this->util->tofloat($item['VRALIQPIS']),
                                               'vrPis'        => $this->util->tofloat($item['VRIMPPIS']),
                                               'bcCofins'     => $this->util->tofloat($item['VRBCPISCOFINS']),
                                               'vrAliqCofins' => $this->util->tofloat($item['VRALIQCOFINS']),
                                               'vrCofins'     => $this->util->tofloat($item['VRIMPCOFINS']),
                                               'bcIcms'       => $this->util->tofloat($item['VRBASECALCICMS']),
                                               'vrAliqIcms'   => $this->util->tofloat($item['VRPEALIMPFIS']),
                                               'vrIcms'       => $this->util->tofloat($item['VRIMPOPRODIT'])
                                           ]
                ];
            array_push($arrayInfo, $itemInfo);
        }
        
        return $arrayInfo;
    }
    
    public function getItens ($subsidiaryCode, $dateSale) {
        $itensOrder = $this->productFactory->getItens($subsidiaryCode, $dateSale, $dateSale);
        if(empty($itensOrder))
            return;
            
        $arrayItens = [];
        $itensEst = [
                    'date'  => $dateSale,
                    'itens' => []
        ];
        
        foreach($itensOrder as $item) {
        
            if($item['IDTIPOCOMPPROD'] == '3' ) {
                $itensCombo = $this->productFactory->getItensCombo($item['CDFILIAL'], $item['CDCAIXA'] , $item['NRSEQVENDA'], $item['NRSEQUITVEND']);
                if(!empty($itensCombo))
                    array_push($arrayItens, $itensCombo);
            }else{
                
                $simpleProduct = [
                    'CDFILIAL'          => $item['CDFILIAL'],
                    'CDCAIXA'           => $item['CDCAIXA'],
                    'NRSEQVENDA'        => $item['NRSEQVENDA'],
                    'NRSEQUITVEND'      => $item['NRSEQUITVEND'],
                    'CDPRODUTO'         => $item['CDPRODUTO'],
                    'NMPRODUTO'         => $item['NMPRODUTO'],
                    'QTITVENDAEST'      => $item['QTPRODVEND'],
                    'IDUTILRETPRODACAB' => $item['IDUTILRETPRODACAB']
                ];
                array_push($arrayItens, $simpleProduct);
                
            }
        }

        foreach($arrayItens as $i) {
            
            if(isset($i['IDUTILRETPRODACAB']) && $i['IDUTILRETPRODACAB'] == 'N') {
                $composicao = $this->productFactory->getComposicao($i['CDFILIAL'], $i['CDPRODUTO']);
                if(!empty($composicao)) {
                   foreach ($composicao as $comp) {
                        $itemComp = array (
                                'name'           => $comp['NMPRODUTO'],
                                'code'           => isset($comp['CDSUBPRODLOC']) ? $comp['CDSUBPRODLOC'] : $comp['CDSUBPRODUTO'],
                                'externalCode'   => '',
                                'unit'           => '',
                                'quantity'       => floatval($this->util->tofloat($i['QTITVENDAEST'])) * (isset($comp['QTSUBPRODLOC']) ? floatval($this->util->tofloat($comp['QTSUBPRODLOC'])) : floatval($this->util->tofloat($comp['QTSUBPRODROT'])))
                            );
                            
                        $product = $this->productFactory->getCodeProduct($itemComp['code']);
                        $itemComp['externalCode'] = $product['CDPRODINTE'];
                        $itemComp['unit'] = $product['SGUNIDADE'];
                        array_push($itensEst['itens'], json_decode(json_encode($itemComp), true));
                   }
                }
            }
            
            if((isset($i['IDUTILRETPRODACAB']) && $i['IDUTILRETPRODACAB'] == 'S') || empty($composicao)) {
                
                $prodPai = array(
                        'name'           => $i['NMPRODUTO'],
                        'code'           => $i['CDPRODUTO'],
                        'externalCode'   => '',
                        'unit'           => '',
                        'quantity'       => floatval($this->util->tofloat($i['QTITVENDAEST']))
                    );
                    
                $product = $this->productFactory->getCodeProduct($prodPai['code']);
                $prodPai['externalCode'] = $product['CDPRODINTE'];
                $prodPai['unit'] = $product['SGUNIDADE'];
                array_push($itensEst['itens'], json_decode(json_encode($prodPai), true));
            }
        }

        return $itensEst;
    }
    
    public function processProduct($cdFilial, $cdLoja, $cdCaixa, $cdOperador, $generalParams, $dtAberCaix, $product, $nrSeqVenda, $nrOrg, $nrSequitVend) {
        
        if((isset($product['code']) && !empty($product['code'])) && empty($product['externalCode'])) {
            $cdProduto = $this->productFactory->getCodeProduct($product['code']);
        }
        
        if(empty($cdProduto) && isset($product['externalCode']) && !empty($product['externalCode'])) {
            $cdProduto = $this->productFactory->getCodeProduct($product['externalCode']);
        }
        
        if(empty($cdProduto)) {
            throw new Exception\ParamsException(
                "O produto {$product['name']} não foi encontrado. Code: {$product['code']} | externalCode: {$product['externalCode']}."); 
        }
        
        $subsidy = isset($produt['subsidizedValue']) ?  $product['subsidizedValue'] : 0;
        $subtotal = bcmul($product['price'], $product['quantity'], 2);
        $total = bcadd($product['addition'], $subtotal, 2);
        $total = bcsub($total, $product['discount'], 2);
        
        $totalWithSubsidy = bcadd($subsidy, $total, 2);
        
        if($total != $product['total']) {
             throw new Exception\ParamsException(
                "Divergência de valores no produto {$product['name']} - Código Teknisa: {$cdProduto['CDARVPROD']}. VrTotal calculado: {$totalWithSubsidy}| VrTotal recebido: {$product['total']}."); 
        }
        
        $cdFamilySald = $this->productFactory->getProductFamily($cdFilial, $cdProduto['CDPRODUTO']);
        
        if(empty($product['fiscalInfo']['ncm'])) {
             throw new Exception\ParamsException(
                "NCM não informado para o produto {$product['name']} - Código Teknisa: {$cdProduto['CDARVPROD']}."); 
        }
        
        $this->productFactory->insertProduct($cdFilial, $cdLoja, $cdCaixa, $cdProduto, $product, $nrSeqVenda, $nrSequitVend, $nrOrg, $cdFamilySald);
        
        if((in_array($product['fiscalInfo']['cstIcms'], self::CST_ICMS_SN) && $generalParams['CDSITUCRT'] != '1')) {
             throw new Exception\ParamsException(
                "Enviado tributação do Simples Nacional porém empresa esta parametrizada como Regime Normal. Produto {$product['name']} - Código Teknisa: {$cdProduto['CDARVPROD']}."); 
        }
        
        if((!in_array($product['fiscalInfo']['cstIcms'], self::CST_ICMS_SN) && $generalParams['CDSITUCRT'] != '1')) {
             throw new Exception\ParamsException(
                "Enviado tributação do Regime Normal para empresa do parametrizada como Simples Nacional. Produto {$product['name']} - Código Teknisa: {$cdProduto['CDARVPROD']}."); 
        }
        
        $productTaxes = $this->productFactory->getProductTaxes($cdFilial, $cdProduto['CDPRODUTO']);
        
        if(empty($productTaxes)) {
             throw new Exception\ParamsException(
                "Impostos não parametrizdos para o produto {$product['name']} - Código Teknisa: {$cdProduto['CDARVPROD']}."); 
        }
        
        $this->productFactory->insertProductTaxes($cdFilial, $cdLoja, $cdCaixa, $cdProduto, $product, $nrSeqVenda, $nrSequitVend, $nrOrg, $productTaxes);
    }
    
    public function getCodeProduct($code) {
        return $this->productFactory->getCodeProduct($code);
    }
    
    public function getProductTaxes($cdFilial, $products) {
        $taxes = $this->productFactory->getTaxesByProducts($cdFilial, $products);
        
        if(empty($taxes)) {
            throw new Exception\ParamsException("Não a impostos cadastrados para o(s) produto(s) informados.");
        }
        
        return $taxes;
    }
    
     public function registerProductTaxes($branchDetails, $taxes, $cdProduto) {
        $getTaxByProduct = $this->productFactory->getProductTaxes($branchDetails['CDFILIAL'], $cdProduto['CDPRODUTO']);
        
        $idTpImposFis = [
            '00' => 'T',
            '99' => 'F',
            '98' => 'I'
        ];
        
        $taxes['IDTPIMPOSFIS'] = $idTpImposFis[$taxes['taxCode']];
        $taxes['icmsCalculationBasis'] =  bcsub($taxes['icmsCalculationBasis'], '1', 0);
        $taxes['taxCode'] == '99' ? $taxes['aliquot'] = 0 : $taxes['aliquot'];
        
        if(empty($getTaxByProduct)) {
            $this->productFactory->registerProductTaxes($branchDetails, $taxes, $cdProduto['CDPRODUTO']);
        } else {
            $this->productFactory->UpdateProductTaxes($branchDetails, $taxes, $cdProduto['CDPRODUTO']);
        }
    }
}